#!/bin/bash
#

# build_larpandoracontent.sh

# if this build script complains that it cannot find bzlib.h, 
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e9:r5|e9:r6|e9:nu:r5|e9:nu:r6> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
    qlist=`echo ${fullqual} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`
    qualver=`echo ${qlist} | tr " " "_"`



}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=larpandoracontent
origpkgver=v02_07_01
pkgver=${origpkgver}
ssibuildshims_version=v0_09_00
cmake_version=v3_3_2
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
lpndver=`echo ${origpkgver} | sed -e 's/_/-/g'`
pkgtarfile=${package}-${lpndver}.tar

get_this_dir

get_ssibuildshims

set_qualifiers

get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

case ${extraqual} in
debug) extra_command="-DCMAKE_BUILD_TYPE=Debug";;
prof)  extra_command="-DCMAKE_BUILD_TYPE=MinSizeRel";;
opt)   extra_command="-DCMAKE_BUILD_TYPE=Release";;
* )
   echo "ERROR: please specify debug, opt, or prof"
   usage
   exit 1
;;
esac

if [[ "${basequal}" == e[79]:r[56] ]] || [[ "${basequal}" == e[79]:nu:r[56] ]]
then
  cxxflg="-std=c++14"
  STDTYPE="BUILD_CXX1Y=1"
else
   echo "ERROR: unrecognized qualifier ${basequal}"
   usage
   exit 1
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || die "ERROR: setup ${package} ${pkgver} failed."
# the build now requires cmake
setup cmake ${cmake_version} -z ${product_dir}:${PRODUCTS} || exit 1

mkdir -p ${LARPANDORACONTENT_FQ_DIR} || die "ERROR: failed to create ${LARPANDORACONTENT_FQ_DIR}"
mkdir -p ${blddir} || die "ERROR: failed to create ${blddir}"

if [ -z ${ROOTSYS} ]
then
   echo "ERROR: failed to setup root"
   exit 1
else
   echo "ROOTSYS: ${ROOTSYS}"
fi
if [ -z ${PANDORA_FQ_DIR} ]
then
   echo "ERROR: failed to setup pandora"
   exit 1
else
   echo "PANDORA_FQ_DIR: ${PANDORA_FQ_DIR}"
fi
pndver=`echo ${PANDORA_VERSION} | sed -e 's/_/-/g' | sed -e 's/[a-z]$//'`

cd ${LARPANDORACONTENT_FQ_DIR}
tar xf ${tardir}/${pkgtarfile} || die "ERROR: failed to unwind ${tardir}/${pkgtarfile}"
#patch -b -p0 < ${PANDORA_DIR}/patch/${pndver}.patch || die "ERROR: patch failed"

set -x
which gcc
cd ${blddir}
# we have to pass the compiler info to the sub-builds
export CC=gcc 
export CXX=g++ 
export FC=gfortran
cmake ${extra_command}  \
     -DCMAKE_MODULE_PATH=${PANDORA_FQ_DIR}/pandora-${pndver}/cmakemodules \
     -DPANDORA_MONITORING=ON \
     -DPandoraSDK_DIR=${PANDORA_FQ_DIR} \
     -DPandoraMonitoring_DIR=${PANDORA_FQ_DIR} \
     -DCMAKE_INSTALL_PREFIX=${LARPANDORACONTENT_FQ_DIR} \
     -DLAR_CONTENT_LIBRARY_NAME=LArPandoraContent \
     -DCMAKE_CXX_FLAGS="${cxxflg}" \
     ${LARPANDORACONTENT_FQ_DIR}/${package}-${lpndver}
make -j${ncore} install
set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: build failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup -B ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS} || die "ERROR: setup failed."
echo "${package} is installed at ${LARPANDORACONTENT_FQ_DIR}"

# fix the cmake package Config files
# ${LARPANDORACONTENT_FQ_DIR}/*.cmake
# ${LARPANDORACONTENT_FQ_DIR}/lib/cmake/*.cmake

fixfile=/tmp/larpandoracontentfix$$$$.sh
rm -f ${fixfile}

echo "#!/bin/bash" > ${fixfile}
##echo " set -x " >>  ${fixfile}

cmakeConfig=`ls ${LARPANDORACONTENT_FQ_DIR}/*.cmake ${LARPANDORACONTENT_FQ_DIR}/lib/cmake/*.cmake`
for file in ${cmakeConfig}
do
  echo "mv ${file} ${file}.bak" >> ${fixfile}
  echo "cat ${file}.bak | sed 's%${LARPANDORACONTENT_FQ_DIR}%\$ENV{LARPANDORACONTENT_FQ_DIR}%g' | sed 's%${PANDORA_FQ_DIR}%\$ENV{PANDORA_FQ_DIR}%g' | sed 's%${ROOTSYS}%\$ENV{ROOTSYS}%g' > ${file}" >> ${fixfile}
done

chmod +x ${fixfile}
${fixfile} || die "ERROR: problem running ${fixfile}."

rm -f ${fixfile}
rm -f ${LARPANDORACONTENT_FQ_DIR}/*.cmake.bak


# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
